@@ -25,6 +25,7 @@ from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_log import log
 from oslo_utils import excutils
+from oslo_utils import strutils
 
 from kuryr.lib import constants as lib_const
 from kuryr.lib import exceptions
@@ -39,12 +40,22 @@ from kuryr_libnetwork import utils
 
 LOG = log.getLogger(__name__)
 
-
 MANDATORY_NEUTRON_EXTENSION = "subnet_allocation"
 TAG_NEUTRON_EXTENSION = "tag"
 TAG_EXT_NEUTRON_EXTENSION = "tag-ext"
 SUBNET_POOLS_V4 = []
 SUBNET_POOLS_V6 = []
+DEFAULT_DRIVER = driver.get_driver_instance()
+try:
+    SRIOV_DRIVER = driver.get_driver_instance(name='sriov')
+except exceptions.KuryrException:
+    SRIOV_DRIVER = None
+VNIC_TYPES_DRIVERS_MAPPING = {
+    const.VNIC_TYPE_NORMAL: DEFAULT_DRIVER,
+    const.VNIC_TYPE_DIRECT: SRIOV_DRIVER,
+    const.VNIC_TYPE_MACVTAP: SRIOV_DRIVER,
+    const.VNIC_TYPE_DIRECT_PHYSICAL: SRIOV_DRIVER,
+}
 
 
 def get_neutron_client():
@@ -97,10 +108,20 @@ def load_default_subnet_pools():
 
 
 def load_port_driver():
-    app.driver = driver.get_driver_instance()
+    app.driver = DEFAULT_DRIVER
     LOG.debug("Using port driver '%s'", str(app.driver))
 
 
+def get_driver(port):
+    vnic_type = port.get('binding:vnic_type', const.VNIC_TYPE_NORMAL)
+    driver = VNIC_TYPES_DRIVERS_MAPPING.get(vnic_type)
+    if driver is None:
+        raise exceptions.KuryrException(
+            "No port driver available for VNIC type %s" % vnic_type)
+    else:
+        return driver
+
+
 def _cache_default_subnetpool_ids(app):
     """Caches IDs of the default subnetpools as app.DEFAULT_POOL_IDS."""
     if not hasattr(app, 'DEFAULT_POOL_IDS'):
@@ -166,6 +187,19 @@ def _get_subnets_by_interface_cidr(neutron_network_id,
     return subnets
 
 
+def _get_subnet_by_name(subnet_name):
+    subnets_by_name = _get_subnets_by_attrs(name=subnet_name)
+    if not subnets_by_name:
+        raise exceptions.NoResourceException(
+            "The subnet doesn't exist for the name {0}"
+            .format(subnets_by_name))
+    elif len(subnets_by_name) > 1:
+        raise exceptions.DuplicatedResourceException(
+            "Multiple Neutron subnets exist for name {0}"
+            .format(subnet_name))
+    return subnets_by_name[0]['id']
+
+
 def _get_neutron_port_from_docker_endpoint(endpoint_id):
     port_name = utils.get_neutron_port_name(endpoint_id)
     filtered_ports = app.neutron.list_ports(name=port_name)
@@ -229,7 +263,7 @@ def _get_fixed_ips_by_interface_cidr(subnets, interface_cidrv4,
 
 
 def _create_or_update_port(neutron_network_id, endpoint_id,
-        interface_cidrv4, interface_cidrv6, interface_mac):
+                           interface_cidrv4, interface_cidrv6, interface_mac):
     subnets = []
     fixed_ips = []
     response_port = []
@@ -252,18 +286,19 @@ def _create_or_update_port(neutron_network_id, endpoint_id,
             .format(interface_cidrv4, interface_cidrv6))
 
     _get_fixed_ips_by_interface_cidr(subnets, interface_cidrv4,
-        interface_cidrv6, fixed_ips)
+                                     interface_cidrv6, fixed_ips)
     filtered_ports = app.neutron.list_ports(fixed_ips=fixed_ips)
     num_port = len(filtered_ports.get('ports', []))
     if not num_port:
         fixed_ips = (
             lib_utils.get_dict_format_fixed_ips_from_kv_format(fixed_ips))
         response_port = _create_port(endpoint_id, neutron_network_id,
-            interface_mac, fixed_ips)
+                                     interface_mac, fixed_ips)
     elif num_port == 1:
         port = filtered_ports['ports'][0]
-        response_port = app.driver.update_port(port, endpoint_id,
-                                               interface_mac)
+        port_driver = get_driver(port)
+        response_port = port_driver.update_port(port, endpoint_id,
+                                                interface_mac)
     # For the container boot from dual-net, request_address will
     # create two ports(v4 and v6 address), we should only allow one
     # for port bind.
@@ -275,7 +310,7 @@ def _create_or_update_port(neutron_network_id, endpoint_id,
         fixed_ips = (
             lib_utils.get_dict_format_fixed_ips_from_kv_format(fixed_ips))
         response_port = _create_port(endpoint_id, neutron_network_id,
-            interface_mac, fixed_ips)
+                                     interface_mac, fixed_ips)
     else:
         raise exceptions.DuplicatedResourceException(
             "Multiple ports exist for the cidrs {0} and {1}"
@@ -475,7 +510,7 @@ def _get_cidr_from_subnetpool(**kwargs):
             .format(kwargs))
 
 
-def _update_existing_port(existing_port, fixed_ip):
+def _update_existing_port(existing_port, fixed_ip, mac_address):
     host = existing_port.get('binding:host_id')
     vif_type = existing_port.get('binding:vif_type')
     if not host and vif_type == 'unbound':
@@ -484,6 +519,8 @@ def _update_existing_port(existing_port, fixed_ip):
             'admin_state_up': True,
             'binding:host_id': lib_utils.get_hostname(),
         }
+        if mac_address:
+            updated_port['mac_address'] = mac_address
         updated_port_resp = app.neutron.update_port(
             existing_port['id'],
             {'port': updated_port})
@@ -496,7 +533,7 @@ def _update_existing_port(existing_port, fixed_ip):
             raise exceptions.AddressInUseException(
                 "Requested ip address {0} already belongs to "
                 "a bound Neutron port: {1}".format(fixed_ip,
-                existing_port['id']))
+                                                   existing_port['id']))
 
     return existing_port
 
@@ -549,9 +586,13 @@ def _create_kuryr_subnet(pool_cidr, subnet_cidr, pool_id, network_id, gateway):
     LOG.debug("Created kuryr subnet %s", new_kuryr_subnet)
 
 
-def _create_kuryr_subnetpool(pool_cidr):
+def _create_kuryr_subnetpool(pool_cidr, pool_tag, shared):
     pool_name = lib_utils.get_neutron_subnetpool_name(pool_cidr)
-    pools = _get_subnetpools_by_attrs(name=pool_name)
+
+    kwargs = {'name': pool_name}
+    if pool_tag:
+        kwargs['tags'] = [pool_tag]
+    pools = _get_subnetpools_by_attrs(**kwargs)
     if len(pools):
         raise exceptions.KuryrException(
             "Another pool with same cidr exist. ipam and network"
@@ -561,11 +602,15 @@ def _create_kuryr_subnetpool(pool_cidr):
     new_subnetpool = {
         'name': pool_name,
         'default_prefixlen': cidr.prefixlen,
-        'prefixes': [pool_cidr]}
+        'prefixes': [pool_cidr],
+        'shared': shared
+    }
     LOG.info("Creating subnetpool with the given pool CIDR")
     created_subnetpool_response = app.neutron.create_subnetpool(
         {'subnetpool': new_subnetpool})
     pool = created_subnetpool_response['subnetpool']
+    if pool_tag:
+        _neutron_subnetpool_add_tag(pool['id'], pool_tag)
     return pool
 
 
@@ -745,10 +790,23 @@ def network_driver_create_network():
     v4_pool_id = ''
     v6_pool_name = ''
     v6_pool_id = ''
+    v4_subnet_name = ''
+    v4_subnet_id = ''
+    v6_subnet_name = ''
+    v6_subnet_id = ''
+    shared = False
     options = json_data.get('Options')
     if options:
         generic_options = options.get(const.NETWORK_GENERIC_OPTIONS)
         if generic_options:
+            v4_subnet_id = generic_options.get(
+                const.NEUTRON_SUBNET_UUID_OPTION)
+            v4_subnet_name = generic_options.get(
+                const.NEUTRON_SUBNET_NAME_OPTION)
+            v6_subnet_id = generic_options.get(
+                const.NEUTRON_V6_SUBNET_UUID_OPTION)
+            v6_subnet_name = generic_options.get(
+                const.NEUTRON_V6_SUBNET_NAME_OPTION)
             neutron_uuid = generic_options.get(const.NEUTRON_UUID_OPTION)
             neutron_name = generic_options.get(const.NEUTRON_NAME_OPTION)
             v4_pool_name = generic_options.get(const.NEUTRON_POOL_NAME_OPTION)
@@ -758,13 +816,19 @@ def network_driver_create_network():
                 const.NEUTRON_POOL_UUID_OPTION)
             v6_pool_id = generic_options.get(
                 const.NEUTRON_V6_POOL_UUID_OPTION)
+            shared = strutils.bool_from_string(generic_options.get(
+                const.NEUTRON_SHARED_OPTION, 'False'))
 
-    def _get_pool_id(pool_name, pool_cidr):
+    def _get_pool_id(pool_name, pool_cidr, pool_tags):
         pool_id = ''
+        kwargs = {}
+        if pool_tags:
+            kwargs['tags'] = pool_tags
         if not pool_name and pool_cidr:
             pool_name = lib_utils.get_neutron_subnetpool_name(pool_cidr)
         if pool_name:
-            pools = _get_subnetpools_by_attrs(name=pool_name)
+            kwargs['name'] = pool_name
+            pools = _get_subnetpools_by_attrs(**kwargs)
             if pools:
                 pool_id = pools[0]['id']
             else:
@@ -779,15 +843,21 @@ def network_driver_create_network():
             raise exceptions.KuryrException(
                 ("Specified pool id({0}) does not "
                  "exist.").format(pool_id))
-
+    if v4_subnet_name and not v4_subnet_id:
+        v4_subnet_id = _get_subnet_by_name(v4_subnet_name)
     if v4_pool_id:
         _verify_pool_id(v4_pool_id)
     else:
-        v4_pool_id = _get_pool_id(v4_pool_name, v4_pool_cidr)
+        v4_pool_tags = [v4_subnet_id] if v4_subnet_id else None
+        v4_pool_id = _get_pool_id(v4_pool_name, v4_pool_cidr, v4_pool_tags)
+
+    if v6_subnet_name and not v6_subnet_id:
+        v6_subnet_id = _get_subnet_by_name(v6_subnet_name)
     if v6_pool_id:
         _verify_pool_id(v6_pool_id)
     else:
-        v6_pool_id = _get_pool_id(v6_pool_name, v6_pool_cidr)
+        v6_pool_tags = [v6_subnet_id] if v6_subnet_id else None
+        v6_pool_id = _get_pool_id(v6_pool_name, v6_pool_cidr, v6_pool_tags)
 
     # let the user override the driver default
     if not neutron_uuid and not neutron_name:
@@ -801,7 +871,8 @@ def network_driver_create_network():
     if not neutron_uuid and not neutron_name:
         network = app.neutron.create_network(
             {'network': {'name': neutron_network_name,
-                         "admin_state_up": True}})
+                         "admin_state_up": True,
+                         'shared': shared}})
         network_id = network['network']['id']
         _neutron_net_add_tags(network['network']['id'], container_net_id,
                               tags=app.tag)
@@ -823,6 +894,7 @@ def network_driver_create_network():
                     ("Specified network id/name({0}) does not "
                      "exist.").format(specified_network))
             network_id = networks[0]['id']
+            network_shared = networks[0]['shared']
         except n_exceptions.NeutronClientException as ex:
             LOG.error("Error happened during listing "
                       "Neutron networks: %s", ex)
@@ -838,6 +910,15 @@ def network_driver_create_network():
                      "%(neutron_network_name)s successfully: %(network)s",
                      {'neutron_network_name': neutron_network_name,
                       'network': network})
+        if network_shared != shared:
+            raise exceptions.ConflictConfigOption(
+                'Network %(network_id)s had option '
+                'shared=%(network_shared)s, conflict with the given option '
+                'shared=%(shared)s', {
+                    'network_id': network_id,
+                    'network_shared': network_shared,
+                    'shared': shared
+                })
         LOG.info("Using existing network %s "
                  "successfully", specified_network)
 
@@ -1054,7 +1135,8 @@ def network_driver_create_endpoint():
             neutron_network_id, endpoint_id, interface_cidrv4,
             interface_cidrv6, interface_mac)
         try:
-            (stdout, stderr) = app.driver.create_host_iface(
+            port_driver = get_driver(neutron_port)
+            (stdout, stderr) = port_driver.create_host_iface(
                 endpoint_id, neutron_port, subnets, filtered_networks[0])
             LOG.debug(stdout)
             if stderr:
@@ -1090,6 +1172,10 @@ def network_driver_create_endpoint():
         if not interface_mac:
             response_interface['MacAddress'] = neutron_port['mac_address']
 
+        vnic_type = neutron_port.get('binding:vnic_type')
+        if vnic_type in const.VNIC_TYPES_SRIOV:
+            response_interface.pop('MacAddress', None)
+
         if not (interface_cidrv4 or interface_cidrv6):
             if 'ip_address' in neutron_port:
                 _process_interface_address(
@@ -1098,6 +1184,8 @@ def network_driver_create_endpoint():
                 _process_interface_address(
                     fixed_ip, subnets_dict_by_id, response_interface)
 
+        LOG.debug("Response JSON data %s for /NetworkDriver.CreateEndpoint",
+                  {'Interface': response_interface})
         return flask.jsonify({'Interface': response_interface})
 
 
@@ -1126,6 +1214,8 @@ def network_driver_endpoint_operational_info():
     response_port_status = (
         _get_neutron_port_status_from_docker_endpoint(endpoint_id))
 
+    LOG.debug("Response JSON data %s for /NetworkDriver.EndpointOperInfo",
+              {'Value': response_port_status})
     return flask.jsonify({'Value': response_port_status})
 
 
@@ -1170,7 +1260,8 @@ def network_driver_delete_endpoint():
         neutron_port = filtered_ports[0]
 
         try:
-            stdout, stderr = app.driver.delete_host_iface(
+            port_driver = get_driver(neutron_port)
+            stdout, stderr = port_driver.delete_host_iface(
                 endpoint_id, neutron_port)
             LOG.debug(stdout)
             if stderr:
@@ -1262,7 +1353,8 @@ def network_driver_join():
                 "Multiple Kuryr subnets exist for the network_id={0} "
                 .format(neutron_network_id))
 
-        iface_name = app.driver.get_container_iface_name(neutron_port['id'])
+        port_driver = get_driver(neutron_port)
+        iface_name = port_driver.get_container_iface_name(neutron_port)
 
         join_response = {
             "InterfaceName": {
@@ -1291,6 +1383,8 @@ def network_driver_join():
                 }
                 join_response['StaticRoutes'].append(static_route)
 
+        LOG.debug("Response JSON data %s for /NetworkDriver.Join",
+                  join_response)
         return flask.jsonify(join_response)
 
 
@@ -1368,7 +1462,7 @@ def ipam_get_capabilities():
       https://github.com/docker/libnetwork/blob/master/docs/ipam.md#getcapabilities  # noqa
     """
     LOG.debug("Received /IpamDriver.GetCapabilities")
-    capabilities = {'RequiresMACAddress': False}
+    capabilities = {'RequiresMACAddress': True}
     return flask.jsonify(capabilities)
 
 
@@ -1428,12 +1522,21 @@ def ipam_request_pool():
     subnet_cidr = ''
     pool_name = ''
     pool_id = ''
+    subnet_id = ''
+    subnet_name = ''
+    shared = False
     options = json_data.get('Options')
     if options:
+        shared = strutils.bool_from_string(options.get(
+            const.NEUTRON_SHARED_OPTION, 'False'))
         if v6:
+            subnet_name = options.get(const.NEUTRON_V6_SUBNET_NAME_OPTION)
+            subnet_id = options.get(const.NEUTRON_V6_SUBNET_UUID_OPTION)
             pool_name = options.get(const.NEUTRON_V6_POOL_NAME_OPTION)
             pool_id = options.get(const.NEUTRON_V6_POOL_UUID_OPTION)
         else:
+            subnet_name = options.get(const.NEUTRON_SUBNET_NAME_OPTION)
+            subnet_id = options.get(const.NEUTRON_SUBNET_UUID_OPTION)
             pool_name = options.get(const.NEUTRON_POOL_NAME_OPTION)
             pool_id = options.get(const.NEUTRON_POOL_UUID_OPTION)
     if requested_pool:
@@ -1442,13 +1545,18 @@ def ipam_request_pool():
         else:
             cidr = ipaddress.ip_network(six.text_type(requested_pool))
         subnet_cidr = six.text_type(cidr)
-        subnets_by_cidr = _get_subnets_by_attrs(cidr=subnet_cidr)
-        if len(subnets_by_cidr):
-            LOG.warning("There is already existing subnet for the "
-                        "same cidr. Please check and specify pool name "
-                        "in Options.")
+        if not subnet_id and subnet_name:
+            subnet_id = _get_subnet_by_name(subnet_name)
+        elif not subnet_id and not subnet_name:
+            subnets_by_cidr = _get_subnets_by_attrs(cidr=subnet_cidr)
+            if len(subnets_by_cidr):
+                LOG.warning("There is already existing subnet for the "
+                            "same cidr. Please check and specify pool name "
+                            "in Options.")
         if not pool_name and not pool_id:
-            pool_id = _create_kuryr_subnetpool(subnet_cidr)['id']
+            pool_id = _create_kuryr_subnetpool(subnet_cidr,
+                                               subnet_id,
+                                               shared)['id']
         else:
             if pool_id:
                 existing_pools = _get_subnetpools_by_attrs(id=pool_id)
@@ -1457,7 +1565,7 @@ def ipam_request_pool():
             if not existing_pools:
                 raise exceptions.KuryrException(
                     ("Specified subnetpool id/name({0}) does not "
-                    "exist.").format(pool_id or pool_name))
+                     "exist.").format(pool_id or pool_name))
 
             pool_id = existing_pools[0]['id']
             if app.tag_ext:
@@ -1466,10 +1574,18 @@ def ipam_request_pool():
             prefixes = existing_pools[0]['prefixes']
             pool_cidr = ipaddress.ip_network(six.text_type(prefixes[0]))
             if pool_cidr == cidr:
+                if shared != existing_pools[0]['shared']:
+                    raise exceptions.ConflictConfigOption(
+                        'There is already existing subnet pool '
+                        'with %(cidr)s but with shared = %(shared)s',
+                        {'cidr': cidr,
+                         'shared': existing_pools[0]['shared']})
                 LOG.info("Using existing Neutron subnetpool %s successfully",
                          pool_id)
             else:
-                pool_id = _create_kuryr_subnetpool(subnet_cidr)['id']
+                pool_id = _create_kuryr_subnetpool(subnet_cidr,
+                                                   subnet_id,
+                                                   shared)['id']
     else:
         if v6:
             default_pool_list = SUBNET_POOLS_V6
@@ -1481,6 +1597,8 @@ def ipam_request_pool():
 
     req_pool_res = {'PoolID': pool_id,
                     'Pool': subnet_cidr}
+    LOG.debug("Response JSON data %s for /IpamDriver.RequestPool",
+              req_pool_res)
     return flask.jsonify(req_pool_res)
 
 
@@ -1514,12 +1632,14 @@ def ipam_request_address():
     jsonschema.validate(json_data, schemata.REQUEST_ADDRESS_SCHEMA)
     pool_id = json_data['PoolID']
     req_address = json_data['Address']
+    req_mac_address = ''
     is_gateway = False
     allocated_address = ''
     subnet = {}
     # Check if the port is gateway
     options = json_data.get('Options')
     if options:
+        req_mac_address = options.get(const.DOCKER_MAC_ADDRESS_OPTION)
         request_address_type = options.get(const.REQUEST_ADDRESS_TYPE)
         if request_address_type == const.NETWORK_GATEWAY_OPTIONS:
             is_gateway = True
@@ -1581,23 +1701,29 @@ def ipam_request_address():
                     'admin_state_up': True,
                     'network_id': neutron_network_id,
                 }
+                if req_mac_address:
+                    port['mac_address'] = req_mac_address
                 fixed_ips = port['fixed_ips'] = []
                 fixed_ip = {'subnet_id': subnet['id']}
-                num_ports = 0
+                filtered_ports = []
                 if req_address:
                     fixed_ip['ip_address'] = req_address
                     fixed_ip_existing = [('subnet_id=%s' % subnet['id'])]
                     fixed_ip_existing.append('ip_address='
                                              '%s' % str(req_address))
                     filtered_ports = app.neutron.list_ports(
-                        fixed_ips=fixed_ip_existing)
-                    num_ports = len(filtered_ports.get('ports', []))
+                        fixed_ips=fixed_ip_existing).get('ports', [])
+                if not filtered_ports:
+                    filtered_ports = app.neutron.list_ports(
+                        mac_address=req_mac_address).get('ports', [])
+
+                num_ports = len(filtered_ports)
                 fixed_ips.append(fixed_ip)
 
                 if num_ports:
-                    existing_port = filtered_ports['ports'][0]
-                    created_port = _update_existing_port(existing_port,
-                                                         fixed_ip)
+                    existing_port = filtered_ports[0]
+                    created_port = _update_existing_port(
+                        existing_port, fixed_ip, req_mac_address)
                     # REVISIT(yedongcan) For tag-ext extension not
                     # supported, the Neutron existing port still can not
                     # be deleted in ipam_release_address.
@@ -1613,8 +1739,11 @@ def ipam_request_address():
                                               lib_const.DEVICE_OWNER)
 
                 LOG.debug("created port %s", created_port)
+                fixed_ips = created_port['fixed_ips']
+                fixed_ips = [ip for ip in fixed_ips
+                             if ip['subnet_id'] == subnet['id']]
                 allocated_address = (req_address or
-                    created_port['fixed_ips'][0]['ip_address'])
+                                     fixed_ips[0]['ip_address'])
                 allocated_address = '{}/{}'.format(allocated_address,
                                                    subnet_cidr.prefixlen)
             except n_exceptions.NeutronClientException as ex:
@@ -1630,6 +1759,8 @@ def ipam_request_address():
             allocated_address = '{}/{}'.format(req_address,
                                                subnet_cidr.prefixlen)
 
+    LOG.debug("Response JSON data %s for /IpamDriver.RequestAddress",
+              {'Address': allocated_address})
     return flask.jsonify({'Address': allocated_address})
 
 
@@ -1741,8 +1872,8 @@ def ipam_release_address():
         for port in all_ports['ports']:
             tags = port.get('tags', [])
             if ((tags and lib_const.DEVICE_OWNER in tags) or
-                (not tags and port['name'] ==
-                 utils.get_neutron_port_name(port['device_id']))):
+                    (not tags and port['name'] ==
+                        utils.get_neutron_port_name(port['device_id']))):
                 for tmp_subnet in subnets:
                     if (port['fixed_ips'][0]['subnet_id'] == tmp_subnet['id']):
                         app.neutron.delete_port(port['id'])
