@@ -177,7 +177,10 @@ class TestVlanDriver(base.TestKuryrBase):
         mock_vlan_check.return_value = None
         vlan_driver = vlan.VlanDriver()
         fake_neutron_port_id = uuidutils.generate_uuid()
-        response = vlan_driver.get_container_iface_name(fake_neutron_port_id)
+        fake_neutron_port = self._get_fake_port(
+            uuidutils.generate_uuid(), uuidutils.generate_uuid(),
+            fake_neutron_port_id)['port']
+        response = vlan_driver.get_container_iface_name(fake_neutron_port)
         mock_get_pair_names.assert_called_with(fake_neutron_port_id)
         self.assertEqual(response, "fake_container_name")
 
@@ -299,7 +302,6 @@ class TestVlanDriver(base.TestKuryrBase):
                 {'port': {
                     'name': fake_port_name,
                     'device_owner': lib_const.DEVICE_OWNER,
-                    'device_id': fake_endpoint_id,
                     'binding:host_id': lib_utils.get_hostname(),
                     'mac_address': fake_neutron_mac_address2
                 }})
