@@ -53,7 +53,7 @@ class TestKuryrIpam(base.TestKuryrBase):
           "GlobalDefaultAddressSpace":
           config.CONF.global_default_address_space}),
         ('/IpamDriver.GetCapabilities',
-         {"RequiresMACAddress": False}))
+         {"RequiresMACAddress": True}))
     @ddt.unpack
     def test_remote_ipam_driver_endpoint(self, endpoint, expected):
         response = self.app.post(endpoint)
@@ -61,6 +61,197 @@ class TestKuryrIpam(base.TestKuryrBase):
         decoded_json = jsonutils.loads(response.data)
         self.assertEqual(expected, decoded_json)
 
+    @mock.patch('kuryr_libnetwork.controllers.app.neutron.add_tag')
+    @mock.patch('kuryr_libnetwork.controllers.app.neutron.create_subnetpool')
+    @mock.patch('kuryr_libnetwork.controllers.app.neutron.list_subnetpools')
+    @ddt.data((FAKE_IP4_CIDR), (FAKE_IP6_CIDR))
+    def test_ipam_driver_request_pool_with_existing_subnet_id(self,
+            pool_cidr, mock_list_subnetpools,
+            mock_create_subnetpool, mock_add_tag):
+        neutron_subnet_v4_id = uuidutils.generate_uuid()
+
+        pool_name = lib_utils.get_neutron_subnetpool_name(pool_cidr)
+        prefixlen = ipaddress.ip_network(six.text_type(pool_cidr)).prefixlen
+        new_subnetpool = {
+            'name': pool_name,
+            'default_prefixlen': prefixlen,
+            'prefixes': [pool_cidr],
+            'shared': False}
+
+        fake_kuryr_subnetpool_id = uuidutils.generate_uuid()
+        fake_name = pool_name
+        if pool_cidr == FAKE_IP4_CIDR:
+            kuryr_subnetpools = self._get_fake_v4_subnetpools(
+                fake_kuryr_subnetpool_id, prefixes=[pool_cidr],
+                name=fake_name)
+        else:
+            kuryr_subnetpools = self._get_fake_v6_subnetpools(
+                fake_kuryr_subnetpool_id, prefixes=[pool_cidr],
+                name=fake_name)
+        mock_list_subnetpools.return_value = {'subnetpools': []}
+        fake_subnetpool_response = {
+            'subnetpool': kuryr_subnetpools['subnetpools'][0]
+        }
+
+        mock_create_subnetpool.return_value = fake_subnetpool_response
+
+        fake_request = {
+            'AddressSpace': '',
+            'Pool': pool_cidr,
+            'SubPool': '',  # In the case --ip-range is not given
+            'Options': {
+                'neutron.subnet.uuid': neutron_subnet_v4_id
+            },
+            'V6': False
+        }
+        response = self.app.post('/IpamDriver.RequestPool',
+                                 content_type='application/json',
+                                 data=jsonutils.dumps(fake_request))
+
+        self.assertEqual(200, response.status_code)
+        mock_list_subnetpools.assert_called_with(
+            name=pool_name, tags=[str(neutron_subnet_v4_id)])
+        mock_create_subnetpool.assert_called_with(
+            {'subnetpool': new_subnetpool})
+        mock_add_tag.assert_called_once_with(
+            'subnetpools', fake_kuryr_subnetpool_id, neutron_subnet_v4_id)
+        decoded_json = jsonutils.loads(response.data)
+        self.assertEqual(fake_kuryr_subnetpool_id, decoded_json['PoolID'])
+
+    @mock.patch('kuryr_libnetwork.controllers.app.neutron.add_tag')
+    @mock.patch('kuryr_libnetwork.controllers.app.neutron.create_subnetpool')
+    @mock.patch('kuryr_libnetwork.controllers.app.neutron.list_subnetpools')
+    @ddt.data((FAKE_IP4_CIDR), (FAKE_IP6_CIDR))
+    def test_ipam_driver_request_pool_with_existing_subnet_id_and_shared(self,
+            pool_cidr, mock_list_subnetpools,
+            mock_create_subnetpool, mock_add_tag):
+        neutron_subnet_v4_id = uuidutils.generate_uuid()
+
+        pool_name = lib_utils.get_neutron_subnetpool_name(pool_cidr)
+        prefixlen = ipaddress.ip_network(six.text_type(pool_cidr)).prefixlen
+        new_subnetpool = {
+            'name': pool_name,
+            'default_prefixlen': prefixlen,
+            'prefixes': [pool_cidr],
+            'shared': True}
+
+        fake_kuryr_subnetpool_id = uuidutils.generate_uuid()
+        fake_name = pool_name
+        if pool_cidr == FAKE_IP4_CIDR:
+            kuryr_subnetpools = self._get_fake_v4_subnetpools(
+                fake_kuryr_subnetpool_id, prefixes=[pool_cidr],
+                name=fake_name)
+        else:
+            kuryr_subnetpools = self._get_fake_v6_subnetpools(
+                fake_kuryr_subnetpool_id, prefixes=[pool_cidr],
+                name=fake_name)
+        mock_list_subnetpools.return_value = {'subnetpools': []}
+        fake_subnetpool_response = {
+            'subnetpool': kuryr_subnetpools['subnetpools'][0]
+        }
+
+        mock_create_subnetpool.return_value = fake_subnetpool_response
+
+        fake_request = {
+            'AddressSpace': '',
+            'Pool': pool_cidr,
+            'SubPool': '',  # In the case --ip-range is not given
+            'Options': {
+                'neutron.subnet.uuid': neutron_subnet_v4_id,
+                'neutron.net.shared': True
+            },
+            'V6': False
+        }
+        response = self.app.post('/IpamDriver.RequestPool',
+                                 content_type='application/json',
+                                 data=jsonutils.dumps(fake_request))
+
+        self.assertEqual(200, response.status_code)
+        mock_list_subnetpools.assert_called_with(
+            name=pool_name, tags=[str(neutron_subnet_v4_id)])
+        mock_create_subnetpool.assert_called_with(
+            {'subnetpool': new_subnetpool})
+        mock_add_tag.assert_called_once_with(
+            'subnetpools', fake_kuryr_subnetpool_id, neutron_subnet_v4_id)
+        decoded_json = jsonutils.loads(response.data)
+        self.assertEqual(fake_kuryr_subnetpool_id, decoded_json['PoolID'])
+
+    @mock.patch('kuryr_libnetwork.controllers.app.neutron.add_tag')
+    @mock.patch('kuryr_libnetwork.controllers.app.neutron.create_subnetpool')
+    @mock.patch('kuryr_libnetwork.controllers.app.neutron.list_subnetpools')
+    @mock.patch('kuryr_libnetwork.controllers.app.neutron.list_subnets')
+    @ddt.data((FAKE_IP4_CIDR), (FAKE_IP6_CIDR))
+    def test_ipam_driver_request_pool_with_existing_subnet_name(self,
+            pool_cidr, mock_list_subnets, mock_list_subnetpools,
+            mock_create_subnetpool, mock_add_tag):
+        # faking list_subnets
+        docker_endpoint_id = lib_utils.get_hash()
+        neutron_network_id = uuidutils.generate_uuid()
+        neutron_subnet_v4_id = uuidutils.generate_uuid()
+        neutron_subnet_v4_name = utils.make_subnet_name(FAKE_IP4_CIDR)
+
+        # Faking existing Neutron subnets
+        fake_v4_subnet = self._get_fake_v4_subnet(
+            neutron_network_id, docker_endpoint_id,
+            subnet_v4_id=neutron_subnet_v4_id,
+            cidr=FAKE_IP4_CIDR, name=neutron_subnet_v4_name)
+
+        fake_subnets = {
+            'subnets': [
+                fake_v4_subnet['subnet'],
+            ]
+        }
+
+        mock_list_subnets.return_value = fake_subnets
+        pool_name = lib_utils.get_neutron_subnetpool_name(pool_cidr)
+        prefixlen = ipaddress.ip_network(six.text_type(pool_cidr)).prefixlen
+        new_subnetpool = {
+            'name': pool_name,
+            'default_prefixlen': prefixlen,
+            'prefixes': [pool_cidr],
+            'shared': False}
+
+        fake_kuryr_subnetpool_id = uuidutils.generate_uuid()
+        fake_name = pool_name
+        if pool_cidr == FAKE_IP4_CIDR:
+            kuryr_subnetpools = self._get_fake_v4_subnetpools(
+                fake_kuryr_subnetpool_id, prefixes=[pool_cidr],
+                name=fake_name)
+        else:
+            kuryr_subnetpools = self._get_fake_v6_subnetpools(
+                fake_kuryr_subnetpool_id, prefixes=[pool_cidr],
+                name=fake_name)
+        mock_list_subnetpools.return_value = {'subnetpools': []}
+        fake_subnetpool_response = {
+            'subnetpool': kuryr_subnetpools['subnetpools'][0]
+        }
+
+        mock_create_subnetpool.return_value = fake_subnetpool_response
+
+        fake_request = {
+            'AddressSpace': '',
+            'Pool': pool_cidr,
+            'SubPool': '',  # In the case --ip-range is not given
+            'Options': {
+                'neutron.subnet.name': neutron_subnet_v4_name
+            },
+            'V6': False
+        }
+        response = self.app.post('/IpamDriver.RequestPool',
+                                content_type='application/json',
+                                data=jsonutils.dumps(fake_request))
+
+        self.assertEqual(200, response.status_code)
+        mock_list_subnets.assert_called_with(name=neutron_subnet_v4_name)
+        mock_list_subnetpools.assert_called_with(
+            name=pool_name, tags=[str(neutron_subnet_v4_id)])
+        mock_create_subnetpool.assert_called_with(
+            {'subnetpool': new_subnetpool})
+        mock_add_tag.assert_called_once_with(
+            'subnetpools', fake_kuryr_subnetpool_id, neutron_subnet_v4_id)
+        decoded_json = jsonutils.loads(response.data)
+        self.assertEqual(fake_kuryr_subnetpool_id, decoded_json['PoolID'])
+
     @mock.patch('kuryr_libnetwork.controllers.app.neutron.create_subnetpool')
     @mock.patch('kuryr_libnetwork.controllers.app.neutron.list_subnetpools')
     @mock.patch('kuryr_libnetwork.controllers.app.neutron.list_subnets')
@@ -74,7 +265,8 @@ class TestKuryrIpam(base.TestKuryrBase):
         new_subnetpool = {
             'name': pool_name,
             'default_prefixlen': prefixlen,
-            'prefixes': [pool_cidr]}
+            'prefixes': [pool_cidr],
+            'shared': False}
 
         fake_kuryr_subnetpool_id = uuidutils.generate_uuid()
         fake_name = pool_name
@@ -238,7 +430,8 @@ class TestKuryrIpam(base.TestKuryrBase):
         new_subnetpool = {
             'name': pool_name,
             'default_prefixlen': prefixlen,
-            'prefixes': [subnet_cidr]}
+            'prefixes': [subnet_cidr],
+            'shared': False}
 
         fake_kuryr_subnetpool_id = uuidutils.generate_uuid()
         fake_existing_subnetpool_id = uuidutils.generate_uuid()
@@ -464,15 +657,18 @@ class TestKuryrIpam(base.TestKuryrBase):
 
         # faking create_port
         fake_neutron_port_id = uuidutils.generate_uuid()
+        fake_mac_address = 'fa:16:3e:ca:59:88'
         fake_port = base.TestKuryrBase._get_fake_port(
             docker_endpoint_id, neutron_network_id,
             fake_neutron_port_id, lib_const.PORT_STATUS_ACTIVE,
             subnet_v4_id,
-            neutron_subnet_v4_address="10.0.0.5")
+            neutron_subnet_v4_address="10.0.0.5",
+            neutron_mac_address=fake_mac_address)
         port_request = {
             'name': const.KURYR_UNBOUND_PORT,
             'admin_state_up': True,
             'network_id': neutron_network_id,
+            'mac_address': fake_mac_address,
         }
         fixed_ips = port_request['fixed_ips'] = []
         fixed_ip = {'subnet_id': subnet_v4_id}
@@ -483,7 +679,7 @@ class TestKuryrIpam(base.TestKuryrBase):
         fake_request = {
             'PoolID': fake_kuryr_subnetpool_id,
             'Address': '',  # Querying for container address
-            'Options': {}
+            'Options': {const.DOCKER_MAC_ADDRESS_OPTION: fake_mac_address}
         }
         mock_port_add_tag.return_value = None
         response = self.app.post('/IpamDriver.RequestAddress',
@@ -506,6 +702,7 @@ class TestKuryrIpam(base.TestKuryrBase):
     def test_ipam_driver_request_address_when_subnet_not_exist(self,
             mock_list_subnetpools, mock_list_subnets):
         requested_address = '10.0.0.5'
+        fake_mac_address = 'fa:16:3e:ca:59:88'
         fake_kuryr_subnetpool_id = uuidutils.generate_uuid()
         fake_name = lib_utils.get_neutron_subnetpool_name(FAKE_IP4_CIDR)
         kuryr_subnetpools = self._get_fake_v4_subnetpools(
@@ -521,7 +718,7 @@ class TestKuryrIpam(base.TestKuryrBase):
         fake_request = {
             'PoolID': fake_kuryr_subnetpool_id,
             'Address': requested_address,
-            'Options': {}
+            'Options': {const.DOCKER_MAC_ADDRESS_OPTION: fake_mac_address}
         }
         response = self.app.post('/IpamDriver.RequestAddress',
                                 content_type='application/json',
@@ -561,12 +758,14 @@ class TestKuryrIpam(base.TestKuryrBase):
 
         # faking update_port or create_port
         requested_address = '10.0.0.5'
+        fake_mac_address = 'fa:16:3e:ca:59:88'
         fake_neutron_port_id = uuidutils.generate_uuid()
         fake_port = base.TestKuryrBase._get_fake_port(
             docker_endpoint_id, neutron_network_id,
             fake_neutron_port_id, lib_const.PORT_STATUS_ACTIVE,
             subnet_v4_id,
-            neutron_subnet_v4_address=requested_address)
+            neutron_subnet_v4_address=requested_address,
+            neutron_mac_address=fake_mac_address)
 
         fixed_ip_existing = [('subnet_id=%s' % subnet_v4_id)]
         fixed_ip_existing.append('ip_address=%s' % requested_address)
@@ -577,6 +776,7 @@ class TestKuryrIpam(base.TestKuryrBase):
             'name': const.KURYR_UNBOUND_PORT,
             'admin_state_up': True,
             'network_id': neutron_network_id,
+            'mac_address': fake_mac_address,
         }
         fixed_ips = port_request['fixed_ips'] = []
         fixed_ip = {'subnet_id': subnet_v4_id,
@@ -588,7 +788,7 @@ class TestKuryrIpam(base.TestKuryrBase):
         fake_request = {
             'PoolID': fake_kuryr_subnetpool_id,
             'Address': requested_address,
-            'Options': {}
+            'Options': {const.DOCKER_MAC_ADDRESS_OPTION: fake_mac_address}
         }
         mock_port_add_tag.return_value = None
         response = self.app.post('/IpamDriver.RequestAddress',
@@ -598,8 +798,9 @@ class TestKuryrIpam(base.TestKuryrBase):
         self.assertEqual(200, response.status_code)
         mock_list_subnets.assert_called_with(
             subnetpool_id=fake_kuryr_subnetpool_id)
-        mock_list_ports.assert_called_with(
-            fixed_ips=fixed_ip_existing)
+        mock_list_ports.assert_has_calls([
+            mock.call(fixed_ips=fixed_ip_existing),
+            mock.call(mac_address=fake_mac_address)])
         mock_create_port.assert_called_with({'port': port_request})
         if mock_app.tag_ext:
             mock_port_add_tag.assert_called()
@@ -650,13 +851,15 @@ class TestKuryrIpam(base.TestKuryrBase):
         # faking update_port or create_port
         requested_address = '10.0.0.5'
         requested_address_v6 = 'fe80::6'
+        requested_mac_address = 'fa:16:3e:86:a0:fe'
         fake_neutron_port_id = uuidutils.generate_uuid()
         fake_port = base.TestKuryrBase._get_fake_port(
             docker_endpoint_id, neutron_network_id,
             fake_neutron_port_id, lib_const.PORT_STATUS_ACTIVE,
             subnet_v4_id, subnet_v6_id,
             neutron_subnet_v4_address=requested_address,
-            neutron_subnet_v6_address=requested_address_v6)
+            neutron_subnet_v6_address=requested_address_v6,
+            neutron_mac_address=requested_mac_address)
 
         fixed_ip_existing = [('subnet_id=%s' % subnet_v4_id)]
         fixed_ipv6_existing = [('subnet_id=%s' % subnet_v6_id)]
@@ -677,6 +880,7 @@ class TestKuryrIpam(base.TestKuryrBase):
             'name': const.NEUTRON_UNBOUND_PORT,
             'admin_state_up': True,
             'binding:host_id': lib_utils.get_hostname(),
+            'mac_address': requested_mac_address,
         }
         mock_update_port.return_value = fake_port
 
@@ -684,7 +888,7 @@ class TestKuryrIpam(base.TestKuryrBase):
         fake_request = {
             'PoolID': fake_kuryr_subnetpool_id,
             'Address': requested_address,
-            'Options': {}
+            'Options': {const.DOCKER_MAC_ADDRESS_OPTION: requested_mac_address}
         }
         mock_port_add_tag.return_value = None
         response = self.app.post('/IpamDriver.RequestAddress',
@@ -698,7 +902,7 @@ class TestKuryrIpam(base.TestKuryrBase):
         fake_request_2 = {
             'PoolID': fake_kuryr_subnetpool_v6_id,
             'Address': requested_address_v6,
-            'Options': {}
+            'Options': {const.DOCKER_MAC_ADDRESS_OPTION: requested_mac_address}
         }
         response = self.app.post('/IpamDriver.RequestAddress',
                                  content_type='application/json',
@@ -721,6 +925,122 @@ class TestKuryrIpam(base.TestKuryrBase):
         else:
             self.assertEqual(0, mock_port_add_tag.call_count)
 
+    @mock.patch('kuryr_libnetwork.controllers._neutron_port_add_tag')
+    @mock.patch('kuryr_libnetwork.controllers.app.neutron.create_port')
+    @mock.patch('kuryr_libnetwork.controllers.app.neutron.update_port')
+    @mock.patch('kuryr_libnetwork.controllers.app.neutron.list_ports')
+    @mock.patch('kuryr_libnetwork.controllers.app.neutron.list_subnets')
+    @mock.patch('kuryr_libnetwork.controllers.app')
+    @ddt.data((False), (True))
+    def test_ipam_driver_request_specific_mac_address_existing_port(self,
+            use_tag_ext, mock_app, mock_list_subnets, mock_list_ports,
+            mock_update_port, mock_create_port, mock_port_add_tag):
+        mock_app.tag_ext = use_tag_ext
+        # faking list_subnets
+        neutron_network_id = uuidutils.generate_uuid()
+        docker_endpoint_id = lib_utils.get_hash()
+        subnet_v4_id = uuidutils.generate_uuid()
+        subnet_v6_id = uuidutils.generate_uuid()
+        fake_kuryr_subnetpool_id = uuidutils.generate_uuid()
+        fake_kuryr_subnetpool_v6_id = uuidutils.generate_uuid()
+        fake_v4_subnet = self._get_fake_v4_subnet(
+            neutron_network_id, docker_endpoint_id, subnet_v4_id,
+            subnetpool_id=fake_kuryr_subnetpool_id,
+            cidr=FAKE_IP4_CIDR)
+        fake_v6_subnet = self._get_fake_v6_subnet(
+            neutron_network_id, docker_endpoint_id, subnet_v6_id,
+            subnetpool_id=fake_kuryr_subnetpool_v6_id,
+            cidr=FAKE_IP6_CIDR)
+        fake_subnet_response = {
+            'subnets': [
+                fake_v4_subnet['subnet']
+            ]
+        }
+        fake_subnet_response_v6 = {
+            'subnets': [
+                fake_v6_subnet['subnet']
+            ]
+        }
+        mock_list_subnets.side_effect = [
+            fake_subnet_response, fake_subnet_response_v6]
+
+        # faking update_port or create_port
+        fake_address = '10.0.0.5'
+        fake_address_v6 = 'fe80::6'
+        requested_mac_address = 'fa:16:3e:86:a0:fe'
+        fake_neutron_port_id = uuidutils.generate_uuid()
+        fake_port = base.TestKuryrBase._get_fake_port(
+            docker_endpoint_id, neutron_network_id,
+            fake_neutron_port_id, lib_const.PORT_STATUS_ACTIVE,
+            subnet_v4_id, subnet_v6_id,
+            neutron_subnet_v4_address=fake_address,
+            neutron_subnet_v6_address=fake_address_v6,
+            neutron_mac_address=requested_mac_address)
+
+        fixed_ip_existing = [('subnet_id=%s' % subnet_v4_id)]
+        fixed_ipv6_existing = [('subnet_id=%s' % subnet_v6_id)]
+        fixed_ip_existing.append('ip_address=%s' % fake_address)
+        fixed_ipv6_existing.append('ip_address=%s' % fake_address_v6)
+        fake_existing_port = dict(fake_port['port'])
+        fake_existing_port['binding:host_id'] = ''
+        fake_existing_port['binding:vif_type'] = 'unbound'
+        fake_ports_response = {'ports': [fake_existing_port]}
+        fake_existing_port_2 = dict(fake_port['port'])
+        fake_existing_port_2['name'] = const.NEUTRON_UNBOUND_PORT
+        fake_existing_port_2['binding:host_id'] = lib_utils.get_hostname()
+        fake_ports_response_2 = {'ports': [fake_existing_port_2]}
+        mock_list_ports.side_effect = [
+            fake_ports_response, fake_ports_response_2]
+
+        update_port = {
+            'name': const.NEUTRON_UNBOUND_PORT,
+            'admin_state_up': True,
+            'binding:host_id': lib_utils.get_hostname(),
+            'mac_address': requested_mac_address,
+        }
+        mock_update_port.return_value = fake_port
+
+        # Testing container ip allocation
+        fake_request = {
+            'PoolID': fake_kuryr_subnetpool_id,
+            'Address': '',
+            'Options': {const.DOCKER_MAC_ADDRESS_OPTION: requested_mac_address}
+        }
+        mock_port_add_tag.return_value = None
+        response = self.app.post('/IpamDriver.RequestAddress',
+                                 content_type='application/json',
+                                 data=jsonutils.dumps(fake_request))
+
+        self.assertEqual(200, response.status_code)
+        decoded_json = jsonutils.loads(response.data)
+        self.assertEqual(fake_address + '/16', decoded_json['Address'])
+
+        fake_request_2 = {
+            'PoolID': fake_kuryr_subnetpool_v6_id,
+            'Address': '',
+            'Options': {const.DOCKER_MAC_ADDRESS_OPTION: requested_mac_address}
+        }
+        response = self.app.post('/IpamDriver.RequestAddress',
+                                 content_type='application/json',
+                                 data=jsonutils.dumps(fake_request_2))
+
+        self.assertEqual(200, response.status_code)
+        decoded_json = jsonutils.loads(response.data)
+        self.assertEqual(fake_address_v6 + '/64', decoded_json['Address'])
+
+        mock_list_subnets.assert_has_calls([
+            mock.call(subnetpool_id=fake_kuryr_subnetpool_id),
+            mock.call(subnetpool_id=fake_kuryr_subnetpool_v6_id)])
+        mock_list_ports.assert_has_calls([
+            mock.call(mac_address=requested_mac_address),
+            mock.call(mac_address=requested_mac_address)])
+        mock_update_port.assert_called_with(fake_neutron_port_id,
+            {'port': update_port})
+        if mock_app.tag_ext:
+            self.assertEqual(2, mock_port_add_tag.call_count)
+        else:
+            self.assertEqual(0, mock_port_add_tag.call_count)
+
     @mock.patch('kuryr_libnetwork.controllers._neutron_port_add_tag')
     @mock.patch('kuryr_libnetwork.controllers.app.neutron.create_port')
     @mock.patch('kuryr_libnetwork.controllers.app.neutron.list_subnets')
@@ -772,16 +1092,19 @@ class TestKuryrIpam(base.TestKuryrBase):
             fake_subnet_response, fake_subnet_response2]
         # faking create_port
         fake_neutron_port_id = uuidutils.generate_uuid()
+        fake_mac_address = 'fa:16:3e:86:a0:fe'
         fake_port = self._get_fake_port(
             docker_endpoint_id, neutron_network_id,
             fake_neutron_port_id,
             neutron_subnet_v4_id=neutron_subnet_v4_id,
-            neutron_subnet_v4_address="10.0.0.5")
+            neutron_subnet_v4_address="10.0.0.5",
+            neutron_mac_address=fake_mac_address)
         mock_create_port.return_value = fake_port
         port_request = {
             'name': const.KURYR_UNBOUND_PORT,
             'admin_state_up': True,
             'network_id': neutron_network_id,
+            'mac_address': fake_mac_address,
         }
         port_request['fixed_ips'] = []
         fixed_ip = {'subnet_id': neutron_subnet_v4_id}
@@ -791,7 +1114,7 @@ class TestKuryrIpam(base.TestKuryrBase):
         fake_request = {
             'PoolID': fake_kuryr_subnetpool_id,
             'Address': '',  # Querying for container address
-            'Options': {}
+            'Options': {const.DOCKER_MAC_ADDRESS_OPTION: fake_mac_address}
         }
         mock_port_add_tag.return_value = None
         response = self.app.post('/IpamDriver.RequestAddress',
@@ -862,10 +1185,11 @@ class TestKuryrIpam(base.TestKuryrBase):
             fake_subnet_response, fake_subnet_response2]
 
         # Testing container ip allocation
+        fake_mac_address = 'fa:16:3e:86:a0:fe'
         fake_request = {
             'PoolID': fake_kuryr_subnetpool_id,
             'Address': '',  # Querying for container address
-            'Options': {}
+            'Options': {const.DOCKER_MAC_ADDRESS_OPTION: fake_mac_address}
         }
         response = self.app.post('/IpamDriver.RequestAddress',
                                 content_type='application/json',
@@ -926,16 +1250,19 @@ class TestKuryrIpam(base.TestKuryrBase):
         mock_list_subnets.return_value = fake_subnet_response
         # faking create_port
         fake_neutron_port_id = uuidutils.generate_uuid()
+        fake_mac_address = 'fa:16:3e:86:a0:fe'
         fake_port = self._get_fake_port(
             docker_endpoint_id, neutron_network_id,
             fake_neutron_port_id,
             neutron_subnet_v4_id=neutron_subnet_v4_id,
-            neutron_subnet_v4_address="10.0.0.5")
+            neutron_subnet_v4_address="10.0.0.5",
+            neutron_mac_address=fake_mac_address)
         mock_create_port.return_value = fake_port
         port_request = {
             'name': const.KURYR_UNBOUND_PORT,
             'admin_state_up': True,
             'network_id': neutron_network_id,
+            'mac_address': fake_mac_address,
         }
         port_request['fixed_ips'] = []
         fixed_ip = {'subnet_id': neutron_subnet_v4_id}
@@ -945,7 +1272,7 @@ class TestKuryrIpam(base.TestKuryrBase):
         fake_request = {
             'PoolID': fake_kuryr_subnetpool_id,
             'Address': '',  # Querying for container address
-            'Options': {}
+            'Options': {const.DOCKER_MAC_ADDRESS_OPTION: fake_mac_address}
         }
         mock_port_add_tag.return_value = None
         response = self.app.post('/IpamDriver.RequestAddress',
